from django.contrib.auth.models import AbstractUser, Group
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from allauth.socialaccount.models import SocialAccount
from cities_light.models import City, Region, Country
import hashlib


class PoliticalFront(models.Model):
    name = models.CharField(
        _('Name'),
        max_length=255,
        unique=True
    )

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Political Front')
        verbose_name_plural = _('Political Fronts')

class ActivismSchoolWorkplace(models.Model):
    name = models.CharField(
        _('Name'),
        max_length=255,
        unique=True
    )

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Activism School Workplace')
        verbose_name_plural = _('Activism School Workplaces')


class Event(models.Model):
    name = models.CharField(
        _('Name'),
        max_length=255,
        unique=True
    )

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Event')
        verbose_name_plural = _('Events')


class CtbRole(models.Model):
    name = models.CharField(
        _('Name'),
        max_length=255,
        unique=True
    )

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('UJS Manager Role')
        verbose_name_plural = _('UJS Manager Roles')


class User(AbstractUser):

    RACE_CHOICES = (
        ('BLACK', _('Black')),
        ('BROWN', _('Brown')),
        ('WHITE', _('White')),
        ('YELLOW', _('Yellow')),
        ('INDIGENOUS', _('Indigenous')),
        ('DO_NOT_KNOW', _('Do not know')),
        ('UNDECLARED', _('Undeclared')),
    )

    GENDER_CHOICES = (
        ('FEMALE', _('Female')),
        ('MALE', _('Male')),
        ('CIS_FEMALE', _('Cis Female')),
        ('CIS_MALE', _('Cis Male')),
        ('AGENDER', _('Agender')),
        ('GENDERQUEER', _('Genderqueer')),
        ('GENDERFLUID', _('Genderfluid')),
        ('NON-CONFORMIST_GENDER', _('Non conformist gender')),
        ('VARIANT_GENDER', _('Variant gender')),
        ('INTERSEX', _('Intersex')),
        ('NON-BINARY', _('Non binary')),
        ('TRANSGENDERED', _('Transgendered')),
        ('PANGENDER', _('Pangender')),
        ('TRANSSEXUAL_WOMAN', _('Transsexual woman')),
        ('TRANSSEXUAL_MAN', _('Transsexual man')),
        ('TRANSFEMINAL', _('Transfeminal')),
        ('TRANSMASCULINE', _('Transmasculine')),
        ('DO_NOT_KNOW', _('Do not know')),
        ('NONE', _('None')),
        ('OTHER', _('Other'))
    )

    SEXUAL_ORIENTATION_CHOICES = (
        ('LESBICA', _('Lesbian')),
        ('GAY', _('Gay')),
        ('BISSEXUAL', _('Bissexual')),
        ('HETEROSSEXUAL', _('Heterossexual')),
        ('ASSEXUAL', _('Assexual')),
        ('FLUIDO', _('Fluid')),
        ('OTHER', _('Other'))
    )

    biography = models.TextField(_('Biography'), blank=True, null=True)
    # First Name and Last Name do not cover name patterns
    # around the globe.
    name = models.CharField(
        _('User\'s name'),
        blank=True,
        max_length=255,
        null=True
    )
    occupation = models.CharField(
        _('Occupation'),
        null=True,
        blank=True,
        max_length=255
    )
    image = models.ImageField(
        _('Image'),
        blank=True,
        null=True,
        upload_to='profile_images'
    )
    gender = models.CharField(
        _('Gender'),
        null=True,
        choices=GENDER_CHOICES,
        max_length=255,
        blank=True
    )
    gender_other = models.CharField(
        _('Other type of gender'),
        null=True, max_length=255,
        blank=True
    )
    sexual_orientation = models.CharField(
        _('Sexual orientation'),
        null=True, max_length=255,
        # choices=SEXUAL_ORIENTATION_CHOICES,
        blank=True
    )
    sexual_orientation_other = models.CharField(
        _('Other sexual orientation'),
        null=True, max_length=255,
        blank=True
    )
    race = models.CharField(
        _('Race'),
        null=True,
        blank=True,
        choices=RACE_CHOICES,
        max_length=255
    )
    birth_date = models.DateField(
        _('Birth Date'),
        blank=True,
        null=True
    )

    # Contact info
    phone_number = models.CharField(
        _('Phone number'),
        null=True,
        blank=True,
        max_length=255
    )
    phone_number_other = models.CharField(
        _('Other phone number'),
        null=True,
        blank=True,
        max_length=255
    )

    # Localization info
    address = models.CharField(
        _('Address'),
        null=True, max_length=255,
        blank=True
    )
    address_number = models.CharField(
        _('Address number'),
        null=True, max_length=255,
        blank=True
    )
    address_extra = models.CharField(
        _('Extra information for address'),
        null=True, max_length=255,
        blank=True
    )
    neighborhood = models.CharField(
        _('Neighborhood'),
        null=True, max_length=255,
        blank=True
    )
    zipcode = models.CharField(
        _('ZIP Code'),
        null=True, max_length=255,
        blank=True
    )
    city = models.ForeignKey(
        City,
        models.CASCADE,
        verbose_name=_('City'),
        related_name='users',
        null=True,
        blank=True,
    )
    state = models.ForeignKey(
        Region,
        models.CASCADE,
        verbose_name=_('Region'),
        related_name='users',
        null=True,
        blank=True
    )
    country = models.ForeignKey(
        Country,
        models.CASCADE,
        verbose_name=_('Country'),
        related_name='users', 
        null=True,
        blank=True,
    )
    # Social info
    social_facebook = models.CharField(
        _('Facebook'),
        null=True, max_length=255,
        blank=True
    )
    social_twitter = models.CharField(
        _('Twitter'),
        null=True, max_length=255,
        blank=True
    )
    social_skype = models.CharField(
        _('Skype'),
        null=True, max_length=255,
        blank=True
    )
    social_google = models.CharField(
        _('Google'),
        null=True, max_length=255,
        blank=True
    )
    site = models.CharField(
        _('Site'),
        null=True, max_length=255,
        blank=True
    )

    # Political info
    political_front = models.ManyToManyField(
        PoliticalFront,
        verbose_name=_('Political front'),
        related_name='users',
        blank=True
    )
    political_front_other = models.CharField(
        _('Other political front'),
        null=True, max_length=255,
        blank=True
    )
    studies = models.NullBooleanField(
        _('Studies'),
        blank=True
    )
    school = models.CharField(
        _('School'),
        null=True, max_length=255,
        blank=True
    )
    activism_school_workplace = models.ManyToManyField(
        ActivismSchoolWorkplace,
        verbose_name=_('Activism in school and/or workplace'),
        related_name='users',
        blank=True
    )
    activism_school_workplace_other = models.CharField(
        _('Other activism form in school and/or workplace'),
        null=True, max_length=255,
        blank=True
    )
    has_mandate = models.NullBooleanField(
        _('Has a mandate'),
        blank=True
    )
    mandate = models.CharField(
        _('Mandate'),
        null=True, max_length=255,
        blank=True
    )

    # Legacy stuff
    cpf = models.CharField(max_length=11, blank=True, null=True, unique=True)
    institution = models.CharField(max_length=255, blank=True, null=True)
    date_joined = models.DateTimeField(_('Date joined'), auto_now_add=True)
    # picture = models.ImageField(_("Picture"), upload_to=hash_name('user-pictures', 'username'), blank=True)
    site = models.URLField(_('Site'), blank=True)
    accepted_terms = models.BooleanField(_('Accepted terms and condition'), default=False)


    def __str__(self):
        return self.username

    # def __str__(self):
    #     if self.first_name or self.last_name:
    #         return self.get_full_name()
    #     return self.email

    # def get_full_name(self):
    #     full_name = '%s %s' % (self.first_name, self.last_name)
    #     return full_name.strip()
    
    # def get_short_name(self):
    #     return self.first_name

    @property
    def image_url(self):
        if self.image:
            return self.image.url
        social_accounts = SocialAccount.objects.filter(user_id=self.id)

        for account in social_accounts:
            picture = account.get_avatar_url()
            if picture:
                return picture
        return "https://gravatar.com/avatar/{}?s=40&d=mm".format(hashlib.md5(self.email.encode('utf-8')).hexdigest())

    def save(self, *args, **kwargs):

        is_new = self.pk is None

        super().save(*args, **kwargs)

        if is_new and settings.REGISTRATION_DEFAULT_GROUP_NAME:
            try:
                self.groups.add(Group.objects.get(name=settings.REGISTRATION_DEFAULT_GROUP_NAME))
                self.save()
            except Group.DoesNotExist:
                pass

    # def get_picture_url(self):
    #     if not self.picture:
    #         location = "/%s/%s" % (settings.STATIC_URL, 'img/avatar-default.png')
    #     else:
    #         location = "/%s/%s" % (settings.MEDIA_URL, self.picture)
    #     return re.sub('/+', '/', location)

    # def get_picture_thumb_url(self,
    #                           options={'size': (150, 150), 'crop': 'scale'}):
    #     try:
    #         return get_thumbnailer(self.picture).get_thumbnail(options).url
    #     except InvalidImageFormatError as e:
    #         return str(settings.STATIC_URL + 'img/avatar-default.png')

    def email_user(self, subject, message, from_email=None):
        send_mail(subject, message, from_email, [self.email])

    def get_user_type(self):
        if self.is_superuser:
            return "superuser"
        elif self.groups.filter(name='professors').count():
            return "professors"
        elif self.groups.filter(name='students').count():
            return "students"
        return "unidentified"

    @property
    def is_profile_filled(self):
        from timtec.settings import ACCOUNT_REQUIRED_FIELDS as fields

        for field in fields:
            try:
                f = getattr(self, field)
                if not f:
                    return False
            except AttributeError:
                raise AttributeError(_('Invalid attribute: %s' % field))
        return True

    def get_certificates(self):
        from core.models import CourseCertification
        return CourseCertification.objects.filter(course_student__user=self)